"""
Test that make+request is making correct HTTP requests

Uses the awesome httpbin.org to validate responses
"""
import platform

import twilio
from nose.tools import assert_equal, raises
from mock import patch, Mock, ANY
from twilio.rest.exceptions import TwilioRestException
from twilio.rest.resources.base import make_request, make_twilio_request
from twilio.rest.resources.connection import Connection
from twilio.rest.resources.connection import PROXY_TYPE_SOCKS5

get_headers = {
    "User-Agent": "twilio-python/{version} (Python {python_version})".format(
        version=twilio.__version__,
        python_version=platform.python_version(),
    ),
    "Accept-Charset": "utf-8",
    "Accept": "application/json",
}

post_headers = get_headers.copy()
post_headers["Content-Type"] = "application/x-www-form-urlencoded"


@patch('twilio.rest.resources.base.Response')
@patch('httplib2.Http')
def test_get_params(http_mock, response_mock):
    http = Mock()
    http.request.return_value = (Mock(), Mock())
    http_mock.return_value = http
    make_request("GET", "http://httpbin.org/get", params={"hey": "you"})
    http.request.assert_called_with("http://httpbin.org/get?hey=you", "GET",
                                    body=None, headers=None)


@patch('twilio.rest.resources.base.Response')
@patch('httplib2.Http')
def test_get_extra_params(http_mock, response_mock):
    http = Mock()
    http.request.return_value = (Mock(), Mock())
    http_mock.return_value = http
    make_request("GET", "http://httpbin.org/get?foo=bar", params={"hey": "you"})
    http.request.assert_called_with("http://httpbin.org/get?foo=bar&hey=you", "GET",
                                    body=None, headers=None)


@patch('twilio.rest.resources.base.Response')
@patch('httplib2.Http')
def test_resp_uri(http_mock, response_mock):
    http = Mock()
    http.request.return_value = (Mock(), Mock())
    http_mock.return_value = http
    make_request("GET", "http://httpbin.org/get")
    http.request.assert_called_with("http://httpbin.org/get", "GET",
                                    body=None, headers=None)


@patch('twilio.rest.resources.base.Response')
@patch('httplib2.Http')
def test_sequence_data(http_mock, response_mock):
    http = Mock()
    http.request.return_value = (Mock(), Mock())
    http_mock.return_value = http
    make_request(
        "POST",
        "http://httpbin.org/post",
        data={"a_list": ["here", "is", "some", "stuff"]},
    )
    http.request.assert_called_with(
        "http://httpbin.org/post",
        "POST",
        body="a_list=here&a_list=is&a_list=some&a_list=stuff",
        headers=None,
    )


@patch('twilio.rest.resources.base.make_request')
def test_make_twilio_request_headers(mock):
    url = "http://random/url"
    make_twilio_request("POST", url, use_json_extension=True)
    mock.assert_called_with("POST", "http://random/url.json",
                            headers=post_headers)


@raises(TwilioRestException)
@patch('twilio.rest.resources.base.make_request')
def test_make_twilio_request_bad_data(mock):
    resp = Mock()
    resp.ok = False
    resp.return_value = "error"
    mock.return_value = resp

    url = "http://random/url"
    make_twilio_request("POST", url)
    mock.assert_called_with("POST", "http://random/url.json",
                            headers=post_headers)


@patch('twilio.rest.resources.base.Response')
@patch('httplib2.Http')
def test_proxy_info(http_mock, resp_mock):
    http = Mock()
    http.request.return_value = (Mock(), Mock())
    http_mock.return_value = http
    Connection.set_proxy_info(
        'example.com',
        8080,
        proxy_type=PROXY_TYPE_SOCKS5,
    )
    make_request("GET", "http://httpbin.org/get")
    http_mock.assert_called_with(timeout=None, ca_certs=ANY, proxy_info=ANY)
    http.request.assert_called_with("http://httpbin.org/get", "GET",
                                    body=None, headers=None)
    proxy_info = http_mock.call_args[1]['proxy_info']
    assert_equal(proxy_info.proxy_host, 'example.com')
    assert_equal(proxy_info.proxy_port, 8080)
    assert_equal(proxy_info.proxy_type, PROXY_TYPE_SOCKS5)



from .credential_lists import (
    CredentialList,
    CredentialLists
)

from .ip_access_control_lists import (
    IpAccessControlList,
    IpAccessControlLists
)

from .origination_urls import (
    OriginationUrl,
    OriginationUrls
)

from .phone_numbers import (
    PhoneNumber,
    PhoneNumbers
)

from .trunks import (
    Trunk,
    Trunks
)



from .. import NextGenInstanceResource, NextGenListResource


class PhoneNumber(NextGenInstanceResource):
    """
    A Phone Number resource.
    See the `TaskRouter API reference
    <https://www.twilio.com/docs/sip-trunking/rest/phone-numbers>_`
    for more information.

    .. attribute:: sid

        The unique ID for this Phone Number.

    .. attribute:: trunk_sid

        The unique ID of the Trunk that owns this Phone Number.
    """

    def delete(self):
        """
        Removes an associated Phone Number from a Trunk.
        """
        return self.parent.delete_instance(self.name)


class PhoneNumbers(NextGenListResource):
    """ A list of Phone Numbers resources """

    name = "PhoneNumbers"
    instance = PhoneNumber
    key = "phone_numbers"

    def list(self, **kwargs):
        """
        Retrieves the list of Phone Number resources for a given trunk sid.
        :param Page: The subset of results that needs to be fetched
        :param PageSize: The size of the Page that needs to be fetched
        """
        return super(PhoneNumbers, self).list(**kwargs)

    def create(self, phone_number_sid):
        """
        Associates a Phone Number with the given Trunk.

        :param phone_number_sid:
        Associates a Phone Number with the given trunk.
        """
        data = {
            'phone_number_sid': phone_number_sid
        }
        return self.create_instance(data)

    def delete(self, sid):
        """
        Disassociates a phone number from the trunk.
        :param sid: Phone Number Sid
        """
        return self.delete_instance(sid)



from .. import NextGenInstanceResource, NextGenListResource


class IpAccessControlList(NextGenInstanceResource):
    """
    An IP Access Control List Resource.
    See the `SIP Trunking API reference
    <https://www.twilio.com/docs/sip-trunking/rest/ip-access-control-lists>_`
    for more information.

    .. attribute:: sid

        The unique ID for this IP Access Control List.

    .. attribute:: trunk_sid

        The unique ID of the Trunk that owns this Credential List.
    """

    def delete(self):
        """
        Disassociate an Ip Access Control List.
        """
        return self.parent.delete_instance(self.name)


class IpAccessControlLists(NextGenListResource):
    """ A list of IP Access Control List resources """

    name = "IpAccessControlLists"
    instance = IpAccessControlList
    key = "ip_access_control_lists"

    def list(self, **kwargs):
        """
        Retrieve the IP Access Control List resources.
        :param Page: The subset of results that needs to be fetched
        :param PageSize: The size of the Page that needs to be fetched
        """
        return super(IpAccessControlLists, self).list(**kwargs)

    def create(self, ip_access_control_list_sid):
        """
        Associate an IP Access Control List with a Trunk.

        :param ip_access_control_list_sid:
            A human readable IP Access Control list sid.
        """
        data = {
            'ip_access_control_list_sid': ip_access_control_list_sid
        }
        return self.create_instance(data)

    def delete(self, ip_access_control_list_sid):
        """
        Disassociate an Ip Access Control List from the Trunk.

        :param ip_access_control_list_sid:
            A human readable IP Access Control list sid.
        """
        return self.delete_instance(ip_access_control_list_sid)



from .. import NextGenInstanceResource, NextGenListResource


class MessagingCountry(NextGenInstanceResource):
    """Pricing information for Twilio Messages in a specific country.

    .. attribute:: iso_country

        The country's 2-character ISO 3166-1 code.

    """
    id_key = "iso_country"


class MessagingCountries(NextGenListResource):
    """A list of countries where Twilio Messages are available.

    The returned list of MessagingCountry objects will not have pricing
    information populated. To get pricing information for a specific country,
    retrieve it with the :meth:`get` method.
    """

    instance = MessagingCountry
    key = "countries"
    name = "Countries"

    def get(self, iso_country):
        """Retrieve pricing information for Twilio Messages in the specified
        country.

        :param iso_country: The two-letter ISO code for the country
        """
        return self.get_instance(iso_country)

    def list(self, **kwargs):
        """Retrieve the list of countries in which Twilio Messages are
        available."""

        return super(MessagingCountries, self).list(**kwargs)



from .. import NextGenInstanceResource, NextGenListResource


class CredentialList(NextGenInstanceResource):
    """
    A Credential List Resource.
    See the `SIP Trunking API reference
    <https://www.twilio.com/docs/sip-trunking/rest/credential-lists>_`
    for more information.

    .. attribute:: sid

        The unique ID for this Credential List.

    .. attribute:: trunk_sid

        The unique ID of the Trunk that owns this Credential List.
    """

    def delete(self):
        """
        Disassociates a Credential List from the trunk.
        """
        return self.parent.delete_instance(self.name)


class CredentialLists(NextGenListResource):
    """ A list of Credential List resources """

    name = "CredentialLists"
    instance = CredentialList
    key = "credential_lists"

    def list(self, **kwargs):
        """
        Retrieve the list of Credential List resources for a given trunk sid.
        :param Page: The subset of results that needs to be fetched
        :param PageSize: The size of the Page that needs to be fetched
        """
        return super(CredentialLists, self).list(**kwargs)

    def create(self, credential_list_sid):
        """
        Associate a Credential List with a Trunk.

        :param credential_list_sid: A human readable Credential list sid.
        """
        data = {
            'credential_list_sid': credential_list_sid
        }
        return self.create_instance(data)

    def delete(self, credential_list_sid):
        """
        Disassociates a Credential List from the Trunk.

        :param credential_list_sid: A human readable Credential list sid.
        """
        return self.delete_instance(credential_list_sid)



from .. import NextGenInstanceResource, NextGenListResource


class OriginationUrl(NextGenInstanceResource):
    """
    An Origination URL resource.
    See the `TaskRouter API reference
    <https://www.twilio.com/docs/sip-trunking/rest/origination-urls>_`
    for more information.

    .. attribute:: sid

        The unique ID for this Origination URL.

    .. attribute:: trunk_sid

        The unique ID of the Trunk that owns this Credential List.
    """

    def delete(self):
        """
        Delete an Origination URL.
        """
        return self.parent.delete_instance(self.name)

    def update(self, **kwargs):
        """
        Update an Origination URL.
        """
        return self.parent.update_instance(self.name, kwargs)


class OriginationUrls(NextGenListResource):
    """ A list of Origination URL resources """

    name = "OriginationUrls"
    instance = OriginationUrl
    key = "origination_urls"

    def create(self, friendly_name, sip_url, priority, weight, enabled):
        """
        Create a Origination URL.

        :param friendly_name: A human readable descriptive text, up to 64
            characters long.
        :param sip_url: The SIP address you want Twilio to route your
            Origination calls to. This must be a sip: schema.
        :param priority: Priority ranks the importance of the URI. Value
            ranges from 0 - 65535.
        :param weight: Weight is used to determine the share of load when
            more than one URI has the same priority.
            Value ranges from 0 - 65535.
        :param enabled: A boolean value indicating whether the URL is
            enabled or disabled.

        """
        data = {
            'friendly_name': friendly_name,
            'sip_url': sip_url,
            'priority': priority,
            'weight': weight,
            'enabled': enabled
        }
        return self.create_instance(data)

    def list(self, **kwargs):
        """
        Retrieve the list of Origination URL resources for a given trunk sid.
        :param Page: The subset of results that needs to be fetched
        :param PageSize: The size of the Page that needs to be fetched
        """
        return super(OriginationUrls, self).list(**kwargs)

    def update(self, origination_url_sid, data):
        """
        Update an Origination Url.

        :param origination_url_sid: A human readable Origination Url sid.
        :param data: Attributes that needs to be updated.
        """
        return self.update_instance(origination_url_sid, data)

    def delete(self, origination_url_sid):
        """
        Delete an Origination Url.

        :param origination_url_sid: A human readable Origination Url sid.
        """
        return self.delete_instance(origination_url_sid)



from .. import NextGenInstanceResource, NextGenListResource


class Trunk(NextGenInstanceResource):
    """
    A Trunk resource.
    See the `TaskRouter API reference
    <https://www.twilio.com/docs/sip-trunking/rest/trunks>_`
    for more information.

    .. attribute:: sid

        The unique ID for this Trunk.
    """

    def delete(self):
        """
        Deletes a Trunk.
        """
        return self.parent.delete_instance(self.name)

    def update(self, **kwargs):
        """
        Updates a Trunk.

        """
        return self.parent.update_instance(self.name, **kwargs)


class Trunks(NextGenListResource):
    """ A list of Trunk resources """

    name = "Trunks"
    instance = Trunk
    key = "trunks"

    def list(self, **kwargs):
        """
        Retrieve the list of Trunk resources.

        :param Page: The subset of results that needs to be fetched
        :param PageSize: The size of the Page that needs to be fetched
        """
        return super(Trunks, self).list(**kwargs)

    def create(self, **kwargs):
        """
        Creates a Trunk.
        """
        return self.create_instance(kwargs)

    def update(self, sid, body):
        """
        Updates a Trunk.
        :param sid: A human readable 34 character unique identifier
        :param body: Request body
        """
        return self.update_instance(sid, body)

    def delete(self, sid):
        """
        Deletes a Trunk.
        :param sid: A human readable 34 character unique identifier
        """
        return self.delete_instance(sid)



import unittest
from mock import patch
from nose.tools import assert_equal
from tests.tools import create_mock_json
from twilio.rest.resources.pricing.messaging_countries import (
    MessagingCountries
)

AUTH = ("AC123", "token")
BASE_URI = "https://pricing.twilio.com/v1"


class MessagingCountriesTest(unittest.TestCase):

    @patch('twilio.rest.resources.base.make_twilio_request')
    def test_messaging_countries(self, request):
        resp = create_mock_json(
            'tests/resources/pricing/messaging_countries_list.json')
        resp.status_code = 200
        request.return_value = resp

        countries = MessagingCountries(BASE_URI + "/Messaging", AUTH)
        result = countries.list()

        assert_equal(result[0].iso_country, "AT")
        assert_equal(len(result), 2)

        request.assert_called_with(
            "GET",
            "{0}/Messaging/Countries".format(BASE_URI),
            auth=AUTH,
            use_json_extension=False,
            params={}
        )

    @patch('twilio.rest.resources.base.make_twilio_request')
    def test_messaging_country(self, request):
        resp = create_mock_json(
            'tests/resources/pricing/messaging_countries_instance.json')
        resp.status_code = 200
        request.return_value = resp

        countries = MessagingCountries(BASE_URI + "/Messaging", AUTH)
        result = countries.get('US')

        assert_equal(result.iso_country, "US")
        assert_equal(result.price_unit, "usd")
        assert_equal(result.outbound_sms_prices[0]['mcc'], "311")
        assert_equal(result.outbound_sms_prices[0]['mnc'], "484")
        assert_equal(result.outbound_sms_prices[0]['carrier'], "Verizon")
        prices = result.outbound_sms_prices[0]['prices']

        assert_equal(prices[0]['number_type'], "mobile")
        assert_equal(prices[0]['base_price'], "0.0075")
        assert_equal(prices[0]['current_price'], "0.0070")

        assert_equal(prices[1]['number_type'], "local")
        assert_equal(prices[1]['base_price'], "0.0075")
        assert_equal(prices[1]['current_price'], "0.0070")

        assert_equal(prices[2]['number_type'], "shortcode")
        assert_equal(prices[2]['base_price'], "0.01")
        assert_equal(prices[2]['current_price'], "0.01")

        assert_equal(prices[3]['number_type'], "toll-free")
        assert_equal(prices[3]['base_price'], "0.0075")
        assert_equal(prices[3]['current_price'], "0.0075")

        inbound_sms_prices = result.inbound_sms_prices

        assert_equal(inbound_sms_prices[0]['number_type'], "local")
        assert_equal(inbound_sms_prices[0]['base_price'], "0.0075")
        assert_equal(inbound_sms_prices[0]['current_price'], "0.0075")

        assert_equal(inbound_sms_prices[1]['number_type'], "shortcode")
        assert_equal(inbound_sms_prices[1]['base_price'], "0.0075")
        assert_equal(inbound_sms_prices[1]['current_price'], "0.005")

        assert_equal(inbound_sms_prices[2]['number_type'], "toll-free")
        assert_equal(inbound_sms_prices[2]['base_price'], "0.0075")
        assert_equal(inbound_sms_prices[2]['current_price'], "0.0075")

        request.assert_called_with(
            "GET",
            "{0}/Messaging/Countries/US".format(BASE_URI),
            auth=AUTH,
            use_json_extension=False,
        )



from twilio.rest.base import TwilioClient
from twilio.rest.resources.trunking import (
    CredentialLists,
    IpAccessControlLists,
    OriginationUrls,
    PhoneNumbers,
    Trunks
)
from twilio.rest.resources import UNSET_TIMEOUT


class TwilioTrunkingClient(TwilioClient):
    """
    A client for accessing the Twilio Trunking API

    :param str account: Your Account SID from `your dashboard
        <https://twilio.com/user/account>`_
    :param str token: Your Auth Token from `your dashboard
        <https://twilio.com/user/account>`_
    :param float timeout: The socket and read timeout for requests to Twilio
    """

    def __init__(self, account=None, token=None,
                 base="https://trunking.twilio.com", version="v1",
                 timeout=UNSET_TIMEOUT):
        """
        Create a Twilio REST API client.
        """
        super(TwilioTrunkingClient, self).__init__(account, token, base,
                                                   version, timeout)
        self.trunk_base_uri = "{0}/{1}/Trunks".format(base, version)

    def credential_lists(self, trunk_sid):
        """
        Return a :class:`CredentialList` instance
        """
        credential_lists_uri = "{0}/{1}/CredentialLists".format(
            self.trunk_base_uri, trunk_sid)
        return CredentialLists(credential_lists_uri, self.auth, self.timeout)

    def ip_access_control_lists(self, trunk_sid):
        """
        Return a :class:`IpAccessControlList` instance
        """
        ip_access_control_lists_uri = "{0}/{1}/IpAccessControlLists".format(
            self.trunk_base_uri, trunk_sid)
        return IpAccessControlLists(ip_access_control_lists_uri, self.auth,
                                    self.timeout)

    def origination_urls(self, trunk_sid):
        """
        Return a :class:`OriginationUrls` instance
        """
        origination_urls_uri = "{0}/{1}/OriginationUrls".format(
            self.trunk_base_uri, trunk_sid)
        return OriginationUrls(origination_urls_uri, self.auth, self.timeout)

    def phone_numbers(self, trunk_sid):
        """
        Return a :class:`PhoneNumbers` instance
        """
        phone_numbers_uri = "{0}/{1}/PhoneNumbers".format(self.trunk_base_uri,
                                                          trunk_sid)
        return PhoneNumbers(phone_numbers_uri, self.auth, self.timeout)

    def trunks(self):
        """
        Return a :class:`Trunks` instance
        """
        return Trunks(self.trunk_base_uri, self.auth, self.timeout)
